@@ -3,10 +3,11 @@ use strict;
 use warnings;
 use Carp ();
 
-our $VERSION = "0.02";
+our $VERSION = "0.03";
 
-use Class::Accessor::Lite rw => [qw(jsonrpc id is_batch is_notification)];
-use JSON::XS;
+use Class::Accessor::Lite rw =>
+  [qw(jsonrpc id is_batch is_notification parsed)];
+use JSON::MaybeXS qw(JSON);
 use Router::Simple;
 use Try::Tiny;
 use JSON::RPC::Spec::Procedure;
@@ -15,9 +16,13 @@ use constant DEBUG => $ENV{PERL_JSON_RPC_SPEC_DEBUG} || 0;
 
 sub new {
     my $class = shift;
-    my $args;
-    $args->{coder} = JSON::XS->new->utf8;
-    $args->{router} = Router::Simple->new;
+    my $args = @_ == 1 ? $_[0] : +{@_};
+    if (!exists $args->{coder}) {
+        $args->{coder} = JSON->new->utf8;
+    }
+    if (!exists $args->{router}) {
+        $args->{router} = Router::Simple->new;
+    }
     $args->{procedure}
       = JSON::RPC::Spec::Procedure->new(router => $args->{router});
     my $self = bless $args, $class;
@@ -26,7 +31,7 @@ sub new {
 }
 
 # universal error routine
-sub error {
+sub _error {
     my ($self, $error) = @_;
     return +{
         jsonrpc => $self->jsonrpc,
@@ -35,7 +40,7 @@ sub error {
     };
 }
 
-sub rpc_invalid_request {
+sub _rpc_invalid_request {
     my ($self) = @_;
     my $error = {
         code    => -32600,
@@ -43,109 +48,136 @@ sub rpc_invalid_request {
     };
     $self->is_notification(undef);
     $self->id(undef);
-    return $self->error($error);
+    return $self->_error($error);
 }
 
-sub rpc_method_not_found {
+sub _rpc_method_not_found {
     my ($self) = @_;
     my $error = {
         code    => -32601,
         message => 'Method not found'
     };
-    return $self->error($error);
+    return $self->_error($error);
 }
 
-sub rpc_invalid_params {
+sub _rpc_invalid_params {
     my ($self) = @_;
     my $error = {
         code    => -32602,
         message => 'Invalid params'
     };
-    return $self->error($error);
+    return $self->_error($error);
 }
 
-sub rpc_internal_error {
+sub _rpc_internal_error {
     my ($self, @args) = @_;
     my $error = {
         code    => -32603,
         message => 'Internal error',
         @args
     };
-    return $self->error($error);
+    return $self->_error($error);
 }
 
-sub rpc_parse_error {
+sub _rpc_parse_error {
     my ($self) = @_;
     my $error = {
         code    => -32700,
         message => 'Parse error'
     };
     $self->id(undef);
-    return $self->error($error);
+    return $self->_error($error);
 }
 
-# parse JSON string
-sub parse {
+sub _parse_json {
     my ($self, $json_string) = @_;
-    warn qq{-- start parsing @{[$json_string]}\n} if DEBUG;
-    my $coder = $self->{coder};
     unless (length $json_string) {
-        return $coder->encode($self->rpc_invalid_request);
+        return $self->_rpc_invalid_request;
     }
 
     # JSON decode
     # rpc call with invalid JSON:
     # rpc call Batch, invalid JSON:
-    my ($request, $has_error);
+    my ($req, $has_error);
     try {
-        $request = $coder->decode($json_string);
+        $req = $self->{coder}->decode($json_string);
     }
     catch {
         $has_error = 1;
-        $request   = $self->rpc_parse_error;
+        $req       = $self->_rpc_parse_error;
     };
     if ($has_error) {
-        return $coder->encode($request);
+        return $req;
     }
+    $self->parsed($req);
+    return;
+}
+
+sub _parse {
+    my ($self) = @_;
+
+    my $req = $self->parsed;
 
     # Batch mode flag
-    if (ref $request eq 'ARRAY') {
+    if (ref $req eq 'ARRAY') {
         $self->is_batch(1);
     }
     else {
         $self->is_batch(0);
-        $request = [$request];
+        $req = [$req];
     }
 
     # rpc call with an empty Array:
-    unless (scalar @{$request}) {
-        return $coder->encode($self->rpc_invalid_request);
+    unless (scalar @{$req}) {
+        return $self->_rpc_invalid_request;
     }
 
     # procedure call and create response
     my @response;
-    for my $obj (@{$request}) {
+    for my $obj (@{$req}) {
         my $res = $self->{procedure}->parse($obj);
 
         # notification is ignore
         push @response, $res if $res;
     }
-    my $result;
     if (@response) {
         if ($self->is_batch) {
-            $result = $coder->encode(\@response);
+            return [@response];
         }
         else {
-            $result = $coder->encode($response[0]);
+            return $response[0];
         }
     }
-    return $result;
+    return;
+}
+
+# parse JSON string to hash
+sub parse_without_encode {
+    my ($self, $json_string) = @_;
+    warn qq{-- start parsing @{[$json_string]}\n} if DEBUG;
+    my $result = $self->_parse_json($json_string);
+    return $result if $result;
+    return $self->_parse;
+}
+
+# parse JSON string to JSON string
+sub parse {
+    my ($self, $json_string) = @_;
+    my $result = $self->parse_without_encode($json_string);
+    return unless $result;
+    return $self->{coder}->encode($result);
 }
 
 # register method
 sub register {
-    my ($self, $name, $cb) = @_;
-    $self->{router}->connect($name, {'.callback' => $cb}, {});
+    my ($self, $pattern, $cb) = @_;
+    if (!defined $pattern) {
+        Carp::croak('pattern required');
+    }
+    if (ref $cb ne 'CODE') {
+        Carp::croak('code required');
+    }
+    $self->{router}->connect($pattern, {'.callback' => $cb}, {});
     return $self;
 }
 
@@ -190,15 +222,30 @@ create instance
     use List::Util qw(max);
     $rpc->register(max => sub { max(@{$_[0]}) });
 
-register method
+    # method matching via Router::Simple
+    $rpc->register('myapp.{action}' => sub {
+        my ($params, $match) = @_;
+        my $action = $match->{action};
+        return MyApp->new->$action($params);
+    });
+
+register method.
 
 =head2 parse
 
     my $result = $rpc->parse(
         '{"jsonrpc": "2.0", "method": "max", "params": [9,4,11,0], "id": 1}'
-    );    # -> {"id":1,"result":11,"jsonrpc":"2.0"}
+    );    # returns JSON encoded string -> {"id":1,"result":11,"jsonrpc":"2.0"}
 
-parse JSON and triggered method
+parse JSON and triggered method. returns JSON encoded string.
+
+=head2 parse_without_encode
+
+    my $result = $rpc->parse_without_encode(
+        '{"jsonrpc": "2.0", "method": "max", "params": [9,4,11,0], "id": 1}'
+    );    # returns hash -> {id => 1, result => 11, jsonrpc => '2.0'}
+
+parse JSON and triggered method. returns HASH.
 
 =head1 DEBUGGING
 
